@@ -1,12 +1,12 @@
 package Locale::MakePhrase::BackingStore::Cached;
-our $VERSION = 0.1;
+our $VERSION = 0.2;
 our $DEBUG = 0;
 
 #
 # This is just a generic caching in-memory backing store, which does
 # no translation whatsoever.  Its really just a lame example...!
 #
-# The returned objects are jsut the requested languages, used with
+# The returned objects are just the requested languages, used with
 # the key.
 #
 
@@ -21,10 +21,10 @@ use base qw(Locale::MakePhrase::BackingStore);
 sub get_rules {
   my ($self,$key,$context,$languages) = @_;
   my @translations;
-  foreach my $lang (@$languages) {
+  foreach my $language (@$languages) {
     my $rule = new Locale::MakePhrase::LanguageRule(
-      language => $lang,
-      translation => "~[$lang~] -> $key",
+      language => $language,
+      translation => "~[$language~] -> $key",
     );
     push @translations, $rule;
   }
