@@ -1,5 +1,5 @@
 package Locale::MakePhrase::Language::en;
-our $VERSION = 0.1;
+our $VERSION = 0.2;
 
 =head1 NAME
 
@@ -12,7 +12,7 @@ English language.
 
 =head1 API
 
-The following functions are implemented:
+The following methods are implemented:
 
 =cut
 
@@ -25,6 +25,19 @@ use Locale::MakePhrase::Utils qw(left);
 
 #--------------------------------------------------------------------------
 
+=head2 $self new([...])
+
+[ Inherited from L<Locale::MakePhrase::Language>. ]
+
+No options are necessary for this module.
+
+If you are using this as an example, you would implement the
+C<init([...])> method, then return a reference to C<$self>.
+
+=cut
+
+#--------------------------------------------------------------------------
+
 =head2 boolean y_or_n($keypress)
 
 Implements handling of B<y> or B<n> keypress for English languages.
