@@ -1,5 +1,5 @@
 package Locale::MakePhrase::Utils;
-our $VERSION = 0.2;
+our $VERSION = 0.4;
 our $DEBUG = 0;
 our $DIE_FROM_CALLER = 0;
 
@@ -35,15 +35,11 @@ use vars qw(@EXPORT_OK);
 
 =head2 boolean is_number(value)
 
-=over 2
-
 Returns true/false indicating if the value is numeric.
 
-=back
-
 =cut
 
-my $is_number_re = qr/^-?(\d+(\.\d*)?|\.\d+)([Ee]-?\d+)?$/;
+my $is_number_re = qr/^-?(?:\d+(?:\.\d*)?|\.\d+)(?:[Ee]-?\d+)?$/;
 
 sub is_number {
   my $value = shift;
@@ -56,12 +52,8 @@ sub is_number {
 
 =head2 string left(string,length)
 
-=over 2
-
 Return the left part of a sub-string.
 
-=back
-
 =cut
 
 sub left {
@@ -72,12 +64,8 @@ sub left {
 
 =head2 string right(string,length)
 
-=over 2
-
 Return the right part of a sub-string.
 
-=back
-
 =cut
 
 sub right {
@@ -88,12 +76,8 @@ sub right {
 
 =head2 string alltrim(string)
 
-=over 2
-
 Trim all leading and trailing whitespace.
 
-=back
-
 =cut
 
 sub alltrim {
@@ -108,20 +92,17 @@ sub alltrim {
 
 =head2 void die_from_caller($message)
 
-=over 2
-
-Throw an exception, from a caller's perspective (ie not from within
+Throw an exception, from a caller's perspective (ie: not from within
 the Locale::MakePhrase modules).  This allows us to generate an error
 message for which the user can figure out what they did wrong.
 
 Note: if you set C<Locale::MakePhrase::Utils::DIE_FROM_CALLER> to a
 value other than zero, die_from_caller() will recurse that number of
 levels further up the stack backtrace, before die()ing.  This allows
-you to wrap your $makePhrase->translate(...) calls in a global
-wrapper function; by setting the value to 1, the message is displayed
-with respect to the calling code.
-
-=back
+you to wrap your $makePhrase-E<gt>translate(...) calls in a global
+wrapper function; eg: by setting the value to 1, the message is
+displayed with respect to one level up of your applications' calling
+code.
 
 =cut
 
@@ -145,6 +126,8 @@ sub die_from_caller {
     }
   }
   $caller = "main" unless defined $caller;
+  $file = "(unknown)" unless defined $file;
+  $line = "(unknown)" unless defined $line;
   my $msg = "Fatal: ". caller() ." detected an error in: $caller\n";
   $msg .= "File: $file\n";
   $msg .= "Line: $line\n";
